using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;


namespace SpriteSplit
{

  public struct SpritePiece
  {
    public Vector2 Position;
    public Vector2 Velocity;
    public Rectangle SourceRectangle;
    public float RotationRate;
    public float Angle;
    public float Age;
    public float MaxAge;
  }


  public class SpriteExploder
  {
    private List<SpritePiece> pieces;
    private Vector2 position;
    private Vector2 origin;

    public List<SpritePiece> Pieces { get { return pieces; } }


    public SpriteExploder(Vector2 position, int spriteWidth, int spriteHeight,
                          int pieceWidth, int pieceHeight)
    {
      Random R = new Random();

      this.position = position;
      this.origin = new Vector2((float)spriteWidth * 0.5f, (float)spriteHeight * 0.5f);
      
      pieces = new List<SpritePiece>();

      // create an instance of each sprite piece
      for (int x = 0; x < spriteWidth; x += pieceWidth)
        for (int y = 0; y < spriteHeight; y += pieceHeight)
        {
          SpritePiece piece = new SpritePiece();

          piece.Position = position + new Vector2(x, y);

          float vx = x - ((float)spriteWidth * 0.5f);
          float vy = y - ((float)spriteHeight * 0.5f);

          piece.Velocity = new Vector2(vx, vy);
          if (piece.Velocity == Vector2.Zero)
            piece.Velocity.X += 0.1f;

          float l = piece.Velocity.Length() * 3.0f;
          piece.Velocity.Normalize();
          piece.Velocity *= (100.0f + l);

          piece.SourceRectangle = new Rectangle(x, y, pieceWidth, pieceHeight);

          piece.Age = 0.0f;
          piece.MaxAge = 25.0f;
          piece.Angle = 0;
          piece.RotationRate = 45.0f * (float)R.NextDouble();

          pieces.Add(piece);
        }

    }


    public void Update(GameTime gameTime)
    {
      float elapsedTime = (float)gameTime.ElapsedGameTime.TotalSeconds;

      for (int i = 0; i < pieces.Count; i++)
      {
        SpritePiece piece = pieces[i];

        // continue on if the piece is dead
        if (piece.Age == -1) continue;

        // update age - if past max age then kill, otherwise update position
        piece.Age += elapsedTime;
        if (piece.Age >= piece.MaxAge)
          piece.Age = -1;
        else
        {
          piece.Angle = (piece.Angle + piece.RotationRate * elapsedTime) % 360.0f;
          piece.Position += piece.Velocity * elapsedTime;
        }

        pieces[i] = piece;
      }
    }


    public void Draw(GameTime gameTime, SpriteBatch spriteBatch, Texture2D spriteTexture)
    {
      foreach(SpritePiece piece in pieces)
        if (piece.Age >= 0)
          spriteBatch.Draw(spriteTexture, piece.Position, piece.SourceRectangle, Color.White,
                           (float)MathHelper.ToRadians(piece.Angle), origin, Vector2.One,
                           SpriteEffects.None, 0);
    }
  }
}